; One Button FOUR Alarm Clock --  ATTINY2313 micro-controller project

; 

; ***** / 3.579545 Xtl for clock


; Chip fuse changes .....
; Uncheck divide by eight fuse -- unit operates at full 3.58 speed
; Set 3-8 MHz 14CK 65 ms CLKSEL=1101 SUT11


; July 4,2006 

.include "tn2313def.inc" 	; ATTINY2313 definition included

; R16 general purpose temporary use register
; R20 contains downcnt.
; R21 contains TPCounter count


;*****Initialization


rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt
rjmp	START1
rjmp	START1

	
START1:			;set up stack pointer and initialize cold start

ldi 	r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16			; Initialize stack pointer to end of internal RAM

ser		r16			;Set to all ones for out port direction as output
out		DDRB,r16	;Set PORTB to output

clr		r16
out		PortB,r16	;LEDs are off when port pins are low

ldi		R16,251		;Set bit2(4)as input ... remaining bits as output
out		DDRD,R16	;execute data direction with bit 0 as input
clr		r16
out		PORTD,R16	;execute to set out ports low, and in port with
					; no input resistor

in		r16,TIMSK	;enable 16-bit match to A register
ori		r16,64		
out		TIMSK,r16

in		r16,TCCR1B
ori		r16,11		; divide by 64 on prescaler
out		TCCR1B,r16	;enable reset of counter to zero (bit3) after match detection.
					;and bits 0,1,2 select prescaling source as 64 of clock

ldi		r16,218
out		OCR1AH,r16
ldi		r16,125		;default selected at 125 for this particular crystal
out		OCR1AL,r16	;loads the compare register A for timer 1 (1 second)
					; -- clock adj. to run slow (for Corr Purp)
			

clr		r14
inc		r14		;put Sunday (1) into Day of Week Counter
clr		r30		;clear minutes counter LSD
clr		r31		;clear minutes counter MSD

clr		r10
clr		r11
clr		r12
clr		r13		;clear alarm number of entries for relays 1-4

clr		r15		;seconds counter
clr		r18		;zero all flags in register r18
clr		r22		;zero all flags in register r22


InitializeClkTime:		;starts clock at 8:00 a.m.

ldi		r16,1				;put Sunday(1) in as start of day-of-week
mov		r14,r16			; put into r14, day of week counter

ldi		R30,224			;8:00 A.M. low byte
ldi		r31,1				;8:00 A.M. high byte

inc		r15				;put one second into seconds counter

ldi		r16,0
sts		$61,r16			;default load of clock correction seconds
	
ldi		r16,1
sts		$62,r16			; $65 =permanent store for interval, 
						; default = 1 = (every day) 
mov		r9,r16			; $66 =temporary store for $65 above, 
						; 0= no correction, 1 = every day
						; 2 = every other day, 3 = every 3rd day, etc.
						; default = add 0 second each day

bset	7				;enable global interrupts


;------------------------------------

MainLoop:


sbis	PIND,2		;skip if bit2(4) set (TP not touched)	
rcall	Decode

sbrc	r22,0		;skip if r22, bit 0(1) is clear - 0/30 or All flag
rjmp	ML4

sbrs	r18,0		;skip if r18,0(1) seconds flag is set
rjmp	MainLoop

cbr		r18,1		;clear seconds flag, r18,0(1)
mov		r16,r15		;monitor seconds count
cpi		r16,30
brne	ML1
rcall	DisplayClock	;flash out time
rjmp	MainLoop


ML1:

mov		r16,r15
cpi		r16,0
brne	MainLoop
rcall	ITenWag			;give long wag for top of the minute
rcall	DisplayClock
rjmp	MainLoop


ML4:					;display clock continuously

rcall	DisplayClock
rcall	DelayOne
rcall	DelayOne		;delay between displays
rjmp	MainLoop


;----------------------------

Decode:				;Monitor Button push and decode

push	r17
push	r20
push	r21
push	r23
push	r25				;save registers used during Button3 & Decode
	
rcall	ButtonPush3		;check for entry

cpi		r20,1			;checks to see if LONG Count has been recorded
brsh	TPHoldDownCount	;branch if same or higher than 1

			
TPShortCount:		;decode short counts


TPSC1:

cpi		r21,1		;Togle AC outlet 1 on/off
brne	TPSC2

rcall	ToggleRelay1
rjmp	TPCEnd


TPSC2:

cpi		r21,2		;Toggle AC outlet 2 on/off
brne	TPSC3

rcall	ToggleRelay2
rjmp	TPCEnd


TPSC3:

cpi		r21,3		;Toggle AC outlet 3 on/off
brne	TPSC4

rcall	ToggleRelay3
rjmp	TPCEnd


TPSC4:

cpi		r21,4
brne	TPCEnd

rcall	ToggleRelay4
rjmp	TPCEnd


;-------------------------------------

TPCEnd:

pop		r25
pop		r23
pop		r21
pop		r20
pop		r17			;restore registers used during button & decode

sbi		PortB,0		;on red led
rcall	DelayQuarter
cbi		PortB,0		;blink red led once
ret


;----------------------------

TPHoldDownCount:		;series for long counts

;-----------------------

HC5:

cpi		r20,5			;Relay 1 commands entry
breq	HC5a
rjmp	HC6


HC5a:

push	r0
push	r16

ldi		r26,$66
clr		r27			;set up start location for Relay 1 storage

ldi		r16,10
mov		r8,r16		;maximum number for permitted entries in r8

rcall	EnterCommands
mov		r10,r0		;move entries count into r10

pop		r16
pop		r0
rjmp	HCEnd


HC6:				;Relay 2 commands entry

cpi		r20,6
breq	HC6a
rjmp	HC7


HC6a:

push	r0
push	r16

ldi		r26,$7A
clr		r27			;set up start location for Relay 2 storage

ldi		r16,10
mov		r8,r16		;maximum number for permitted entries in r8

rcall	EnterCommands
mov		r11,r0		;move entries count into r11

pop		r16
pop		r0
rjmp	HCEnd


HC7:

cpi		r20,7			;Relay 3 commands entry
breq	HC7a
rjmp	HC8


HC7a:

push	r0
push	r16

ldi		r26,$8E
clr		r27			;set up start location for Relay 3 storage

ldi		r16,10
mov		r8,r16		;maximum number for permitted entries in r8

rcall	EnterCommands
mov		r12,r0		;move entries count into r12

pop		r16
pop		r0
rjmp	HCEnd


HC8:

cpi		r20,8		;Relay 4 commands entry
breq	HC8a
rjmp	HC10


HC8a:

push	r0
push	r16

ldi		r26,$A2
clr		r27			;set up start location for Relay 4 storage

ldi		r16,10
mov		r8,r16		;maximum number for permitted entries in r8

rcall	EnterCommands
mov		r13,r0		;move entries count into r13

pop		r16
pop		r0
rjmp	HCEnd


HC10:				;Set DOW and CLOCK

cpi		r20,10
breq	HC10a
rjmp	HC11


HC10a:

push	r16

rcall	SetClock

pop		r16
rjmp	HCEnd


HC11:						;Set LSB of crystal countdown register

cpi		r20,11
breq	HC11a
rjmp	HC12


HC11a:

push	r16

rcall	SetCompareRegister


pop		r16
rjmp	HCEnd


HC12:					;Clock correction at Midnight

cpi		r20,12
breq	HC12a
rjmp	HC13


HC12a:

push	r16

rcall	SecondsCorrection

pop		r16
rjmp	HCEnd


HC13:			;Clear all commands from alarm memory

cpi		r20,13
breq	HC13a
rjmp	HC14


HC13a:

clr		r10
clr		r11
clr		r12
clr		r13			;zero number of memory entries in each category
rjmp	HCEnd


HC14:				;Toggle clock display from 0,30 or continuous

cpi		r20,14
breq	HC14a
rjmp	HCEnd


HC14a:

sbrs	r22,0		;skip if r22,0(1) is set
rjmp	HC14b

cbr		r22,1		;clear r22,0(1) for 0 and 30 min display
rjmp	HCEnd


HC14b:

sbr		r22,1		;set r22,0(1) for continuous display
rjmp	HCEnd


;------------------------------

HCEnd:

rjmp	TPCEnd

;rcall	TenWag

;ret


;---------------------------------------

EnterCommands:		;Register pair r28,29 will accumulate minutes entry

push	r16
push	r19
push	r28
push	r29			;borrow registers for this operation


EC5:

ldi		r16,10
rcall	QuickRedFlash		;warn that you must enter 1 "dit" to continue

rcall	ButtonStall
cpi		r20,0
brne	EC6
rjmp	ECEnd		;exit if short "dit" touch is given. A Hold down
					;  touch will allow one to continue

EC6:

mov		r16,r8		;r8 contains max number of iterations
cpi		r16,10
brne	EC1
clr		r0			;clear entries count, r0, only on first go thru


EC1:

clr		r28
clr		r29			;zero register pair r28,29

ldi		r16,5
rcall	SlowGreenFlash	;flash five times for five entries

rcall	ButtonStall
swap	r20				;swap low bytes to high bytes
lsr		r20				;shift data into position
sts		$63,r20			;put data in $63 scratch store area


rcall	TwoGreenFlash	;signal two entries coming up - MSD, LSD hours
rcall	ButtonStall
rcall	Mult10			;r20 contains value multiplied by ten
mov		r19,r20			;save value in r19

rcall	OneGreenFlash	;signal one more entry needed
rcall	ButtonStall
add		r19,r20			;add r20 to r19 - hours now complete


EC2:

tst		r19
breq	EC3

adiw	r28,60			;add 60 minutes to register pair r26,27
dec		r19
brne	EC2


EC3:

rcall	TwoGreenFlash	;signal two entries needed, MSD LSD minutes
rcall	ButtonStall
rcall	Mult10			;multiply r20, MSD, by ten (minutes)
mov		r19,r20			;store in r19

rcall	OneGreenFlash
rcall	ButtonStall
add		r19,r20			;add in LSD of minutes
add		r28,r19			;add in r19 to r28
brcc	EC4

inc		r29				;increment r29 from carry from addition


EC4:

lds		r16,$63
or		r29,r16			;or in DOW data - pair now complete 

st		x+,r28
st		x+,r29			;data now stored into memory area

inc		r0				;entries count

dec		r8
brne	EC5				;start another entry


ECEnd:

pop		r29
pop		r28
pop		r19
pop		r16

ret


;----------------------------------

ClockCntDwnInt: 		;interrupt to update clock counters

push	r16

in		r16,SREG	;save flags
sts		$60,r16

inc		r15			;step one count of SECONDS counter
mov		r16,r15

sbr		r18,1		;set bits 0(1) -- seconds flag 


CCDI1a:

cpi		r16,60
breq	CCDI1

rjmp	CCDIExit		;return since no action required


CCDI1:

clr		r15			;zero r15 -- SECONDS counter
sbr		r18,2		;set bit 1(2) -- the NEW MINUTE flag

adiw	r30,1		;increment minutes reg pair "z" (r30,31) by one

cpi		r31,5		;compare high byte for match
brne	CCDIalarmExit

cpi		r30,160		;compare low byte for match
brne	CCDIalarmExit


CCDI1c:			;zero high and low bytes - it is midnight

clr		r30
clr		r31			;zero minutes counter

mov		r16,r14
inc		r16			;increment day of week counter
cpi		r16,8		;if >8 then change count
brne	CCDI1cc

ldi		r16,1		;change day count to 1, Sunday


CCDI1cc:

mov		r14,r16		;move correct day of week value into storage


ClockCorrection:	;adds seconds in $61 to time at minight 
					; determined by $62 & $63 (interval)

tst		r9			;if zero, no correction is needed
					;1= every day, 2= every other day, etc.

breq	CCDIalarmExit	;exit routine with no action

dec		r9
brne	CCDIalarmExit

lds		r9,$62		;reload interval
	

CC3:

lds	r16,$61		;seconds correction from permanent store
mov	r15,r16		;put in seconds store area to correct time (add)


CCDIalarmExit:	;CHECK ALL ALARMS ...

CheckForMatch:

push	r20			;scratch
push	r21			;scratch

push	r26
push	r27			;work register "x" pair


CFM1:				;For Relay 1

mov		r20,r10		;move relay 1 command number into r20 work reg
ldi		r26,$66		;low byte for relay one command area
ldi		r27,0		;high byte for relay one 
ldi		r21,10		;limit reads to ten


CFM1a:

tst		r20			
breq	CFM2

rcall	Check		;issue valid commands
sbrc	r18,6		;skip if r18,6(64) is clear
rcall	ToggleRelay1


dec		r21			;limit reads register
breq	CFM2
dec		r20			;command number register
rjmp	CFM1a


CFM2:				;For Relay 2

mov		r20,r11		;move relay 2 command number into r20 work reg
ldi		r26,$7A		;low byte for relay one command area
ldi		r27,0		;high byte for relay one 
ldi		r21,10		;limit reads to ten


CFM2a:

tst		r20			
breq	CFM3

rcall	Check		;issue valid commands
sbrc	r18,6		;skip if r18,6(64) is clear
rcall	ToggleRelay2

dec		r21			;limit reads register
breq	CFM3
dec		r20			;command number register
rjmp	CFM2a


CFM3:				;For Relay 3

mov		r20,r12		;move relay 1 command number into r20 work reg
ldi		r26,$8E		;low byte for relay one command area
ldi		r27,0		;high byte for relay one 
ldi		r21,10		;limit reads to ten


CFM3a:

tst		r20			
breq	CFM4

rcall	Check		;issue valid commands
sbrc	r18,6		;skip if r18,6(64) is clear
rcall	ToggleRelay3

dec		r21			;limit reads register
breq	CFM4
dec		r20			;command number register
rjmp	CFM3a


CFM4:				;For Relay 4

mov		r20,r13		;move relay 1 command number into r20 work reg
ldi		r26,$A2		;low byte for relay one command area
ldi		r27,0		;high byte for relay one 
ldi		r21,10		;limit reads to ten


CFM4a:

tst		r20			
breq	CFMEnd

rcall	Check		;issue valid commands
sbrc	r18,6		;skip if r18,6(64) is clear
rcall	ToggleRelay4

dec		r21			;limit reads register
breq	CFMEnd
dec		r20			;command number register
rjmp	CFM4a


CFMEnd:

pop		r27
pop		r26
pop		r21
pop		r20


CCDIExit:		;Exit this interrupt routine

lds	r16,$60		;SREG temporary store
out	SREG,r16	;restore SREG data

pop	r16

reti


;-----------------------------

Check:			;check for match of data with clock SUBROUTINE

push	r20
push	r21

cbr		r18,64	;clear data match flag r18,6(64)

ld		r20,x+	;load r20 with low byte of alarm data
ld		r21,x+	;load r21 with high byte of alarm data

cp		r20,r30	;check low byte for match
brne	DMEnd

mov		r16,r21		;save r21 into r16 for processing
andi	r16,7		;save 3 LSBits
cp		r16,r31		;Do MSbits of time match alarm data?
brne	DMEnd

; "Does the day data match" in following section
		
mov		r16,r21		;save r21 into r16 for processing
lsr		r16
lsr		r16
lsr		r16			;place DOW data into position

tst		r16
brne	DM1


DMEndSet:

sbr		r18,64		;set data match flag r18,6(64)


DMEnd:

pop		r21
pop 	r20
ret


DM1:

cpi		r16,8		;Is it weekdays only (8) ?
brne	DM2

mov		r20,r14
cpi		r20,7
breq	DM2
cpi		r20,1
breq	DM2

rjmp	DMEndSet


DM2:

cpi		r16,9		;Is it weekends only (9) ?
brne	DM3

mov		r20,r14
cpi		r20,7
breq	DMEndSet
cpi		r20,1
breq	DMEndSet


DM3:

cpi		r16,10		;Is it even days only (10) ?
brne	DM4

sbrs	r14,0		;skip if bit r14,0(1) is set
rjmp	DMEndSet


DM4:

cpi		r16,11		;Is it odd days only (11) ?
brne	DM5
sbrc	r14,0		;skip if bit r14,0(1) is clear
rjmp	DMEndSet


DM5:

cp		r14,r16		;check for match on a day (1-7)
brne	DMEnd

rjmp	DMEndSet


;-----------------------------------------

VarClockFlash:		;flashes red or green LED (G=AM R=PM)
					;R16 contains number of flashes
push	r17

rcall	IDelayHalf
rcall	IDelayOne	;delay before flashes start

					;check if AM or PM time (less than 720 = AM)
cpi		r31,2		;go to AM if high byte is <2 
brlo	VCF1		;jump to AM routine
cpi		r31,2
breq	VCF3
rjmp	VCF2		;jump to PM routine


VCF3:

cpi		r30,208		
brlo	VCF1		;jump to AM routine
rjmp	VCF2		;jump to PM routine


VCF1:				; AM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

sbi		PortB,1		;turn on GREEN LED (AM)
rcall	IDelay40th	;on time
cbi		PortB,1		;off Green LED

sbrc	r18,7		;skip if bit 7 is clear - no double flash
rjmp	VCF1ab		;go for double flash


VCF1ac:

rcall	IDelayHalf		;off time
dec		r16
rjmp	VCF1


VCF1ab:			;gives double flash

rcall	IDelay40th
rcall	IDelay40th	;small pause
sbi		PortB,1		;on green LED
rcall	IDelay40th	;on time
cbi		PortB,1		;off Green LED
rjmp	VCF1ac


VCF2:			; PM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

sbi		PortB,0		;turn on RED LED
rcall	IDelay40th	;on time delay
cbi		PortB,0		;turn off RED LED
sbrc	r18,7		;skip if bit 7 is clear - no double flash
rjmp	VCF2ab


VCF2ac:

rcall	IDelayHalf		;off time delay
dec		r16
rjmp	VCF2


VCF2ab:				;give double red flash

rcall	IDelay40th
rcall	IDelay40th	;short off delay
sbi		PortB,0		;on Red LED
rcall	IDelay40th	;on time delay for second flash
cbi		PortB,0		;off Red LED
rjmp	VCF2ac


VCF2a:

pop		r17
cbr		r18,128		;clear bit 7(128) - double flash flag
ret


;-------------------------------------------

DisplayClock:		;displays time and can be interrupted in IDelays
					;Uses r24 instead of r16 to avoid conflicts 
					;  during interrupts in this routine.


cbr		r18,1		;clear seconds flag, r18,0(1)

push	r6
push	r7
push	r16
push	r28
push	r29

rcall	ITenWag		;begin clock display with wag of LEDs

bclr	7			;turn off interrupts

mov		r28,r30		;load low byte into r28
mov		r29,r31		;load high byte into r29

bset	7			;turn interrupts back on

tst		r29			;test high byte for zero
brne	DC3

cpi		r28,60		;check to see if less than 60 (12:00 am to 12:59 am)
brlo	DC2
rjmp	DC3


DC2:

ldi		r16,12			;flash out 12 a.m.
rcall	VarClockFlash	;flash out time of 12 a.m.

rjmp	DC7


DC3:				;flash out other times

mov		r4,r28		;save low byte in r4
mov		r5,r29		;save high byte in r5

inc		r6			;increment hours count
sbiw	r28,60		;subtract 60 from the r28,29 pair

brcs	DC3a		;hours now almost decoded

rjmp	DC3			;repeat loop again


DC3a:

dec		r6		;restore old value of hours
mov		r28,r4	;restore low byte in r28
				;at this point, minutes remainder is in low byte
				; (r29 will be zero)
			

DC10:

mov		r16,r6		;put hours into r16 for check
cpi		r16,13
brsh	DC9
rjmp	DC8


DC9:

subi	r16,12


DC8:				;flash out time

rcall	VarClockFlash


DC7:				;decode minutes with MSD double flashed

mov		r16,r28
inc		r7			;use r7 for MSD count
subi	r28,10		;get remaining minutes
brcs	DC7a

rjmp	DC7			;loop back for another subtraction


DC7a:

dec		r7			;restore correct value for MSD
mov		r28,r16		;restore correct remainder (LSDs)

mov		r16,r7		;put in MSD for flashout
sbr		r18,128		;set bit 7(128) for double flash
rcall	VarClockFlash	;flash out MSD
cbr		r18,128		;clear bit 7(128) of double flash

mov		r16,r28
rcall	VarClockFlash	;flash out LSDs

rcall	IDelayOne	;delay after display
rcall	IDelayHalf

pop		r29
pop		r28
pop		r16
pop		r7
pop		r6

ret


;---------------------------

SetCompareRegister:

		;puts new value into compare register A.  
		;Default = 218 for high byte, 140 for low byte.
		
push	r1
push	r16

clr		r1
rcall	ThreeGreenFlash

rcall	ButtonStall
rcall	Mult100		;multiply r20 value by 100
mov		r1,r20		;set 100's column

rcall	OneGreenFlash	;indicate an entry is needed
rcall	ButtonStall
rcall	Mult10		;multiply r20 value by 10
add		r1,r20		;add in tens units

rcall	OneGreenFlash	;indicate an entry is needed
rcall	ButtonStall
add		r1,r20		;add in LSD of count -- data is now complete

ldi		r16,218
out 	OCR1AH,r16
out		OCR1AL,r1	;put new compare register A data in place

pop		r16
pop		r1
ret


;-------------------------------------

SecondsCorrection:	;Set seconds of correction and how often (2 entries)

rcall	TwoGreenFlash
rcall	ButtonStall
sts		$61,r20		;seconds of correction (zero to 255)

rcall	OneGreenFlash
rcall	ButtonStall	;enter interval from 0 to 255 - zero = no correction
sts		$62,r20		;put interval into permanent $62 storage
mov		r9,r20		;put interval into temp r9 storage

ret


;---------------------

SetClock:		;sets clock for AM/PM, hour, minutes, and day-of-week
			;  24 hour time entry required

push	r1		;minutes accumulator
push	r2		;hours accumulator
push	r3		;day of week accumulator
push	r16
push	r26
push	r27

clr		r26		;low byte accumulator
clr		r27		;high byte accumulator

ldi		r16,5
rcall	SlowGreenFlash	;indicate that five entries are needed

rcall	ButtonStall
mov		r3,r20		;day of week (1=Sunday, 7=Saturday)

rcall	TwoGreenFlash
rcall	ButtonStall	;enter MSD of hours 

rcall	Mult10		;multiply value by 10 and store in r2
mov		r2,r20

rcall	OneGreenFlash	;signal another entry needed
rcall	ButtonStall		;enter LSD of hours
add		r2,r20			;value in hours now complete in r2

rcall	TwoGreenFlash	;signal another entry needed
rcall	ButtonStall		;enter msd of minutes
rcall	Mult10			;multiply by 10 and store in r1
mov		r1,r20

rcall	OneGreenFlash	;signal another entry needed
rcall	ButtonStall
add		r1,r20			;value in minutes now complete


SetC1:

tst		r2			;total hours count
breq	SetC2		;done
adiw	r26,60
dec		r2
rjmp	SetC1


SetC2:

add		r26,r1		;total minutes count
brcs	SetC3
rjmp	SetEnd


SetC3:

inc		r27			;carry flag was set
rjmp	SetEnd


SetEnd:

bclr	7		;turn off global interrupts

mov		r14,r3
mov		r30,r26		;store low time byte 
mov		r31,r27		;store high time byte

clr		r15			;zero seconds counter

bset	7		;turn on global interrupts

pop		r27
pop		r26
pop		r16
pop		r3
pop		r2
pop		r1

ret


;-------------------------------------

Mult100:		;Multiplies value in R20 by 100 and returns value in R20

push	r1
push	r16

clr		r1			;zero r1 for upcoming data

tst		r20
breq	Mult100b

ldi		r16,100		;the multiplication value


Mult100a:

add		r1,r16
dec		r20
brne	Mult100a


Mult100b:

mov		r20,r1		;for return of value

pop		r16
pop		r1	
ret


;--------------------------------------

Mult10:		; multiplies value in R20 by 10 returning value in R20

push 	r1
push	r16

clr		r1			;zero R1

tst		R20			;test long count value
breq	Mult10b		;value is zero

ldi		r16,10		;put ten into r16

Mult10a:

add		r1,r16		;add ten for each value in R1
dec		r20
brne	Mult10a

Mult10b:

mov	r20,r1		;move r1 value into r16

pop		r16
pop		r1
ret


;---------------------------------------

VariableDelay:	;r16 contains number of 40th second delays desired

rcall	Delay40th
dec		r16
breq	VD2
rjmp	VariableDelay


VD2:

ret


;--------------------------------------

IVariableDelay:	;r16 contains number of interruptible variable 40th delays

rcall	IDelay40th
dec		r16
breq	IVD2
rjmp	IVariableDelay


IVD2:

ret


;----------------------------------------

Delay40th:		;1/40th second delay

push	r0
push	r16
clr		r0
ldi		r16,116


D40a:

dec		r0
brne	D40a

dec		r16
brne	D40a

pop		r16
pop		r0

ret


;------------------------------------------

IDelay40th:			;interruptible 1/40th second delay

push	r0
push	r16

clr		r0
ldi		r16,69


ID40a:

sbis	PIND,2	;skip if switch is up - bit 2(4)
rcall	Decode
dec		r0
brne	ID40a

dec		r16
brne	ID40a

pop		r16
pop		r0

ret


;-----------------------------------------

DelayTenth:

push 	r16
ldi		r16,4


DelayTenth1:

rcall	Delay40th
dec		r16
brne	DelayTenth1

pop		r16
ret


;-----------------------------------------

DelayQuarter:

push	r16
ldi		r16,10

rjmp	DelayTenth1


;-------------------------------------------

DelayHalf:

push 	r16
ldi		r16,20

rjmp	DelayTenth1


;-------------------------------------------

DelayOne:

push	r16
ldi		r16,40

rjmp	DelayTenth1


;-------------------------------------

IDelayTenth:

push	r16
ldi		r16,4


IDelayTenth1:

rcall	IDelay40th
dec		r16
brne	IDelayTenth1

pop		r16
ret


;----------------------------

IDelayQuarter:

push	r16
ldi		r16,10

rjmp	IDelayTenth1


;------------------------------

IDelayHalf:

push	r16
ldi		r16,20

rjmp	IDelayTenth1


;----------------------------------

IDelayOne:

push	r16
ldi		r16,40

rjmp	IDelayTenth1


;----------------------------------------

IThreeWag:

push	r16
ldi		r16,3

rjmp	ITenWag1

;----------------------------------------

ITenWag:				

push	r16
ldi		r16,10


ITenWag1:

sbi		PortB,0		;on red LED
rcall	IDelay40th
cbi		PortB,0		;off red LED
rcall	Delay40th
sbi		PortB,1		;on Green LED
rcall	IDelay40th
cbi		PortB,1
dec		r16
brne	ITenWag1

pop		r16
ret


;--------------------------------------------

ThreeWag:

push	r16
ldi		r16,3

rjmp	TenWag1	


;--------------------------------------------

TenWag:				

push	r16

ldi		r16,10


TenWag1:

sbi		PortB,0		;on red LED
rcall	Delay40th
cbi		PortB,0		;off red LED
rcall	Delay40th
sbi		PortB,1		;on Green LED
rcall	Delay40th
cbi		PortB,1
dec		r16
brne	TenWag

pop		r16
ret


;-------------------------------------------

QuickRedFlash:		;r16 contains number of flashes

sbi		PortB,0		;on RED LED
rcall	Delay40th
cbi		PortB,0		;off RED LED
rcall	Delay40th
dec		r16
brne	QuickRedFlash

ret


;----------------------------------------

SlowRedFlash:		;r16 contains number of flashes

sbi		PortB,0		;On Red LED
rcall	Delay40th
cbi		PortB,0		;Off Red LED
rcall	DelayHalf
dec		r16
brne	SlowRedFlash

ret


;-----------------------------------------

ISlowRedFlash:			;r16 contains number of flashes

sbi		PortB,0
rcall	Delay40th
cbi		PortB,0
rcall	DelayHalf
dec		r16
brne	ISlowRedFlash

ret


;----------------------------------------

SlowGreenFlash:			;r16 contains number of flashes

sbi		PortB,1			;on green LED
rcall	Delay40th
cbi		PortB,1			;off green LED
rcall	DelayHalf
dec		r16
brne	SlowGreenFlash

ret


;--------------------------------------

OneGreenFlash:

ldi		r16,1
rjmp	SlowGreenFlash


;------------------------------

TwoGreenFlash:

ldi		r16,2
rjmp	SlowGreenFlash


;------------------------------

ThreeGreenFlash:

ldi		r16,3
rjmp	SlowGreenFlash


;------------------------------

ToggleRelay1:

sbrs	r18,2		;skip if bit 2(4), relay 1 flag is set
rjmp	TR1a

cbr		r18,4		;clear bit 2(4), relay 1 flag
sbi		PortB,7		;off relay
rcall	Delay40th
cbi		PortB,7		;complete pulse
ret

TR1a:

sbr		r18,4		;set bit 2(4), relay 1 flag
sbi		PortB,6		;on relay
rcall	Delay40th	
cbi		PortB,6		;complete pulse
ret


;---------------------------------------------

ToggleRelay2:

sbrs	r18,3		;skip if bit 3(8), relay 2 flag is set
rjmp	TR2a

cbr		r18,8		;clear bit 3(8), relay 2 flag
sbi		PortB,5		;off relay
rcall	Delay40th
cbi		PortB,5		;complete pulse
ret

TR2a:

sbr		r18,8		;set bit 3(8), relay 2 flag
sbi		PortB,4		;on relay
rcall	Delay40th	
cbi		PortB,4		;complete pulse
ret


;------------------------------------------------

ToggleRelay3:

sbrs	r18,4		;skip if bit 4(16), relay 3 flag is set
rjmp	TR3a

cbr		r18,16		;clear bit 4(16), relay 3 flag
sbi		PortB,3		;off relay
rcall	Delay40th
cbi		PortB,3		;complete pulse
ret

TR3a:

sbr		r18,16		;set bit 4(16), relay 3 flag
sbi		PortB,2		;on relay
rcall	Delay40th	
cbi		PortB,2		;complete pulse
ret


;--------------------------------------

ToggleRelay4:

sbrs	r18,5		;skip if bit 5(32), relay 4 flag is set
rjmp	TR4a

cbr		r18,32		;clear bit 5(32), relay 4 flag
sbi		PortD,6		;off relay
rcall	Delay40th
cbi		PortD,6		;complete pulse
ret

TR4a:

sbr		r18,32		;set bit 5(32), relay 4 flag
sbi		PortD,5		;on relay
rcall	Delay40th	
cbi		PortD,5		;complete pulse
ret


;----------------------------------

ButtonStall:			;Waits for button to be pushed & debounces

Push 	r16


BS1:

clr		r16


BS3:

sbic	PIND,2		;skip if bit 2(4) -- switch is down (on)
rjmp	BS1

rcall	Delay40th
inc		r16

cpi		r16,2
brsh	BSEnd

rjmp	BS3


BSEnd:

pop		r16


;---

ButtonPush3:

;Special Note:  The code register, r17, is not used, nor is the "dah" touch.

cbi		PortB,0		;off Red LED
cbi		PortB,1		;off Green LED

clr		r17			;clear code entry register
clr		r20			;Holddown count
clr		r21			;Dit or dah count
clr		r23			;zero universal delay counter


B1a:

clr		r25			;clear 4 second WAIT counter


B1:

inc		r23			;increment universal delay counter
rcall	Delay40th
cpi		r23,90
brsh	B3

sbis	PIND,2		;skip is switch not down
rjmp	B1a

cpi		r23,20
brsh	B1b			;it is dah count

sbi		PortB,0		;on red LED briefly
rcall	DelayTenth		
cbi		PortB,0
inc		r21			;increment dit count

lsl		r17			;shift ZERO into code register r17, code register


B1c:

clr		r23


B2d:

sbic	PIND,2		;skip if switch down
rjmp	B5

clr		r23
rjmp	B1a


B1b:

sec					;set carry flag
rol		R17			;rotate ONE into code register, r17

inc		r21			;increment short touch count by one

sbi		PortB,0		;short flash of LED
rcall	DelayTenth
cbi		PortB,0		;off LED

rjmp	B1c


B3:

sbi		PortB,0			;on PortB,0(1), red LED
ldi		r16,2
rcall	VariableDelay	;1/20th second on time
cbi		PortB,0			;off LED

inc		r20			;add ONE to Holddown Counter

clr		r23			;clear universal delay counter


B4:

inc		r23
rcall	Delay40th

sbic	PIND,2		;skip if switch down
rjmp	B5

clr		r25			;zero 3 second wait counter
cpi		r23,30	
brsh	B3

rjmp	B4


B5:

;sbrs	r18,0		;skip if r18,0(1) is set (second flag)
;rjmp	B2d

;cbr		r18,1		;clear r18,0(1) second flag




inc		r25			;increment r25, 10second counter



rcall	Delay40th






cpi		r25,60		;limit wait period to 3 seconds
brlo	B2d			;continue

ret


;----------------------------------------------
